############################################################################
# DrawingWindow.py - Leslie Kaelbling                                      #
############################################################################
#    MIT SoaR 1.0 - A Python abstraction layer for MobileRobots            #
#    Pioneer3DX robots, and a simulator to simulate their operation        #
#    in a python environment (for testing)                                 #
#                                                                          #
#    Copyright (C) 2006 Michael Haimes <mhaimes@mit.edu>                   #
#        with code from Ross Glashan <rng@mit.edu>                         #
#                                                                          #
#   This program is free software; you can redistribute it and/or modify   #
#   it under the terms of the GNU General Public License as published by   #
#   the Free Software Foundation; either version 2 of the License, or      #
#   (at your option) any later version.                                    #
#                                                                          #
#   This program is distributed in the hope that it will be useful,        #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
#   GNU General Public License for more details.                           #
#                                                                          #
#   You should have received a copy of the GNU General Public License along#
#   with this program; if not, write to the Free Software Foundation, Inc.,#
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.            #
############################################################################

from Tkinter import *
import soarapp
from soarapp import AppManager

def killDrawingWindows():
	for win in soarapp.DrawingWindows:
		win.destroy()
	DrawingWindows = []

class DrawingWindow:

    def __init__(self, windowWidth, windowHeight, xMin, xMax, yMin, yMax, title="tk"):
        # Make a window to draw on
        top = AppManager.getWindow(title)
     	soarapp.DrawingWindows.append(top) 
	self.windowWidth = windowWidth
        self.windowHeight = windowHeight
        self.w = Canvas(top, width=self.windowWidth, height=self.windowHeight)
        self.w.pack()
	self.destroy = lambda: AppManager.killWindow(title)
        
        # multiply an input value by this to get pixels
        self.xScale = windowWidth / float(xMax - xMin)
        self.yScale = windowHeight / float(yMax - yMin)

        self.xMin = xMin
        self.yMin = yMin
        self.xMax = xMax
        self.yMax = yMax

    def scaleX(self, x):
        return self.xScale * (x - self.xMin)
    
    def scaleY(self, y):
        return self.windowHeight - self.yScale * (y - self.yMin)
        
    def drawPoint(self, x, y, color = "blue"):
        windowX = self.scaleX(x)
        windowY = self.scaleY(y)
        return self.w.create_rectangle(windowX-1, windowY-1,
                                       windowX+1, windowY+1,
                                       fill = color, outline = color)

    def drawRobotWithNose(self, x, y, theta, color = "blue", size = 6):
        (noseX,noseY) = localToGlobal((x,y,theta), (0.15, 0.0))
        self.drawRobot(x,y,noseX,noseY,color=color,size=size)

    def drawRobot(self, x, y, noseX, noseY,color = "blue", size = 8):
        windowX = self.scaleX(x)
        windowY = self.scaleY(y)
        hsize = int(size)/2   # For once, we want the int division!
        return (self.w.create_rectangle(windowX-hsize, windowY-hsize,
                                        windowX+hsize, windowY+hsize,
                                        fill = color, outline = color),
                self.w.create_line(windowX, windowY,
                                   self.scaleX(noseX),self.scaleY(noseY),
                                   fill=color, width=2, arrow="last"))

    def drawRect(self, (x1,y1), (x2,y2), color = "black"):
        self.w.create_rectangle(self.scaleX(x1), self.scaleY(y1),
                                self.scaleX(x2), self.scaleY(y2),
                                fill = color)

    def drawLineSeg(self, x1, y1, x2, y2, color = "black", width = 2):
        return self.w.create_line(self.scaleX(x1),self.scaleY(y1),
                                  self.scaleX(x2),self.scaleY(y2),
                                  fill = color,
                                  width = width)

    def drawUnscaledLineSeg(self, x1, y1, xproj, yproj, color = "black", width = 1):
        return self.w.create_line(self.scaleX(x1),self.scaleY(y1),
                                  self.scaleX(x1)+xproj,self.scaleY(y1)-yproj,
                                  fill = color,
                                  width = width)

    def drawUnscaledRect(self, x1, y1, xproj, yproj, color = "black"):
        self.w.create_rectangle(self.scaleX(x1)-xproj, self.scaleY(y1)+yproj,
                                self.scaleX(x1)+xproj, self.scaleY(y1)-yproj,
                                fill = color)

    def drawLine(self, (a,b,c), color = "black"):
        if abs(b) < 0.001:
            startX = self.scaleX(-c/a)
            endX = self.scaleX(-c/a)
            startY = self.scaleY(self.yMin)
            endY = self.scaleY(self.yMax)
        else:
            startX = self.scaleX(self.xMin)
            startY = self.scaleY(- (a * self.xMin + c) / b)
            endX = self.scaleX(self.xMax)
            endY = self.scaleY(- (a * self.xMax + c) / b)
        return self.w.create_line(startX, startY, endX, endY, fill = color)

    def delete(self, thing):
        self.w.delete(thing)

    def clear(self):
        self.w.delete("all")

