############################################################################
# soarapp.py - Michael Haimes                                              #
############################################################################
#    MIT SoaR 1.0 - A Python abstraction layer for MobileRobots            #
#    Pioneer3DX robots, and a simulator to simulate their operation        #
#    in a python environment (for testing)                                 #
#                                                                          #
#    Copyright (C) 2006 Michael Haimes <mhaimes@mit.edu>                   #
#        with code from Ross Glashan <rng@mit.edu>                         #
#                                                                          #
#   This program is free software; you can redistribute it and/or modify   #
#   it under the terms of the GNU General Public License as published by   #
#   the Free Software Foundation; either version 2 of the License, or      #
#   (at your option) any later version.                                    #
#                                                                          #
#   This program is distributed in the hope that it will be useful,        #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
#   GNU General Public License for more details.                           #
#                                                                          #
#   You should have received a copy of the GNU General Public License along#
#   with this program; if not, write to the Free Software Foundation, Inc.,#
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.            #
############################################################################

from tkMessageBox import showerror, showinfo
from Tkinter import Tk, Toplevel, Label, Button, TclError
import os
import sys
from os.path import dirname

PROGRAM_NAME="SoaR"
WINDOW_ICON="hourglass"

class ApplicationManager:

	def __init__(self):
		self.tk=Tk()
		self.tk.withdraw()
		self.tk.bind("<Control-c>", lambda: self.exit(-1))
		"""
		if os.name == "posix":
			for pid in [line.strip().split()[:1][0] for line in os.popen("ps ax | grep ./soar.py | grep -v grep").readlines()[:-1]+os.popen("ps ax | grep \"python soar.py\" | grep -v grep").readlines()[:-1]]: 
				os.popen("kill " + pid) #kills any hanging old instances of soar
		"""
		self.exitcallbacks=[]
		self.windowkills = {}

	def setRunner(self, runner):
		self.runner = runner

	def getWindow(self, title, nokill=True, nomenuadd=False):
		window = Toplevel(self.tk)
	        window.wm_title(PROGRAM_NAME + ": " + title)
		window.wm_iconbitmap(WINDOW_ICON)
		if not nomenuadd:
			def menucmd():
				window.lift()
				window.focus()
			try:
				self.runner.windowmenu.add_command(label = title, command = menucmd)
			except AttributeError:
				#operating without soar instance, sketchy but allowable
				pass
			def actualdestroy():
				try:
					self.runner.windowmenu.delete(title)
				except TclError:
					#the windowmenu is gone, so don't delete
					pass
				except AttributeError:
					#operationg without soar instance, sketchy but allowable
					pass
				window.destroy()
			self.windowkills[title] = actualdestroy
		else:
			self.windowkills[title] = window.destroy
		if nokill:
      		  	window.protocol('WM_DELETE_WINDOW', self.trytoquit)
		else:
			window.protocol('WM_DELETE_WINDOW', self.windowkills[title])
		return window

	def killWindow(self, title):
		self.windowkills[title]()

	def error(self, msg):
		showerror(parent=self.tk, title = PROGRAM_NAME + ": Error", message = msg)
	
	def alert(self, msg):
		showinfo(parent=self.tk, title = PROGRAM_NAME + ": Alert", message = msg)

	def trytoquit(self):
		sure = Toplevel(self.tk)
		sure.wm_title("Be careful")
		Label(sure, text = "Are you sure you want to exit?").pack(side = "top") 
		Button(sure, text = "Yes, Exit", command = lambda: self.exit(0)).pack(side = "left")
		Button(sure, text = "No!!", command = sure.destroy).pack(side = "right") 

	def exit(self, code):
		for call in self.exitcallbacks:
			call()
		self.tk.destroy()
		"""
		if os.name == "posix":
			linuxkill()
		else:
		"""
		sys.exit(code)

AppManager = ApplicationManager()
DrawingWindows = []

class Container:
        def __init__(self):
                pass

def linuxkill():
	os.popen("kill " + os.popen("ps ax | grep soar.py").readlines()[0].split()[0])


def skip(): pass
def pwd():
	try:
		dir = os.getcwd()
	except:
		if os.name == "posix":
			dir = os.popen("pwd").readline()[:-1]
		elif os.name == "nt":
			dir = os.popen("cd").readline()[:-1]
	return dir

#parse a python file in a given environment, and get out some results
def parseFile(filename, envin, envout = [], notthere = skip):
	exec "from sys import path" in envin
	exec "path.append(\"" + dirname(filename) + "\")" in envin
	envin.pop("path")
	lines = open(filename, 'r').readlines()
	if len(lines) > 0 and '\r' in lines[0]: 
		print "hello"
		lines = [line[:-2]+line[-1:] for line in lines]
	linestr = ""
	for line in lines: linestr+=line
	exec linestr in envin
	if len(envout) > 0:
		returndict = {}
		for key in envout:
			try:
				returndict[key] = envin[key]
			except KeyError:
				returndict[key] = notthere
		return returndict
