############################################################################
# soarmath.py - Michael Haimes                                             #
############################################################################
#    MIT SoaR 1.0 - A Python abstraction layer for MobileRobots            #
#    Pioneer3DX robots, and a simulator to simulate their operation        #
#    in a python environment (for testing)                                 #
#                                                                          #
#    Copyright (C) 2006 Michael Haimes <mhaimes@mit.edu>                   #
#        with code from Ross Glashan <rng@mit.edu>                         #
#                                                                          #
#   This program is free software; you can redistribute it and/or modify   #
#   it under the terms of the GNU General Public License as published by   #
#   the Free Software Foundation; either version 2 of the License, or      #
#   (at your option) any later version.                                    #
#                                                                          #
#   This program is distributed in the hope that it will be useful,        #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
#   GNU General Public License for more details.                           #
#                                                                          #
#   You should have received a copy of the GNU General Public License along#
#   with this program; if not, write to the Free Software Foundation, Inc.,#
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.            #
############################################################################

from math import acos, sqrt, pi
from random import uniform

#return a number in a string  (guaranteed max length of 4 + length of appropriate power of 10) 
#representation of its value in scientific notation
def scinot(num):
    string=`num`
    power = 0
    i = 0
    j = string[i]
    str = ''
    while not j =='.' and i < len(string):
        j = string[i]
        str+=j
        power+=1
        i+=1
    i-=1
    if i < 5:
        str = string[:5]
    if i >= 5 and power > 0:
        str= string[:4] + 'e' + `power`
    return str

#call apply() on every possible composition of one element from each list
def argFor(vec, apply, current = []):
    if len(vec) == 0:
        apply(current)
    else:
        for i in vec[0]:
            next = current + [i]
            argFor(vec[1:], apply, next)

sq = lambda x: x*x
def norm(vec):
	div = sqrt(float(sum(map(sq, vec))))
        return tuple(map(lambda x: x/div, vec))
dot = lambda veca, vecb: sum([veca[i]*vecb[i] for i in range(len(veca))])
vec = lambda line: (line[1][0]-line[0][0], line[1][1]-line[0][1])
angle = lambda linea, lineb: acos(dot(norm(vec(linea)), norm(vec(lineb))))
clip = lambda val, vn, vx: min(vx, max(vn, val))

#simple approximation for whether sonar should return or not based on angle
def angleok(angle):
	angle*=180/pi
#bubble wrap?	return 1-sq(sq(angle/90.0)) > uniform(0, 1)
	if angle < 30: return 1-(angle**6)/(2*(30.**6)) > uniform(0,1)
	else: return ((angle-90)**6)/(2*(60.**6)) > uniform(0,1)

def intersectionhelper(((xa, ya),(xb,yb)),((xc,yc),(xd,yd))):
        try:
        	xa, ya, xb, yb, xc, yc, xd, yd = tuple(map(float, [xa, ya, xb, yb, xc, yc, xd, yd]))
                s = ((xb-xa)*(ya-yc)+(yb-ya)*(xc-xa))/((xb-xa)*(yd-yc)-(yb-ya)*(xd-xc)) 
                t = ((xc-xa)+(xd-xc)*s)/(xb-xa)
                if s <= 1 and s >=0 and t <= 1 and t >= 0:
                	fromt = (xa+(xb-xa)*t,ya+(yb-ya)*t)
                        froms = (xc+(xd-xc)*s,yc+(yd-yc)*s)
                        if abs(fromt[0]-froms[0])<0.001 and abs(fromt[1]-froms[1])<0.001:
                        	return fromt
                return False 
        except ZeroDivisionError:
                return False

#returns the point of intersection between two line segments, or False if there is no intersection
def intersection(linea, lineb):

        first = intersectionhelper(linea, lineb)
        if first:
                return first
        else:
                return intersectionhelper(lineb, linea)
