############################################################################
# brain.py - Michael Haimes                                                #
############################################################################
#    MIT SoaR 1.0 - A Python abstraction layer for MobileRobots            #
#    Pioneer3DX robots, and a simulator to simulate their operation        #
#    in a python environment (for testing)                                 #
#                                                                          #
#    Copyright (C) 2006 Massachusetts Institute of Technology              #
#      Written by Michael Haimes <mhaimes@mit.edu>                         #
#        with code from Ross Glashan <rng@mit.edu>                         #
#                                                                          #
#   This program is free software; you can redistribute it and/or modify   #
#   it under the terms of the GNU General Public License as published by   #
#   the Free Software Foundation; either version 2 of the License, or      #
#   (at your option) any later version.                                    #
#                                                                          #
#   This program is distributed in the hope that it will be useful,        #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
#   GNU General Public License for more details.                           #
#                                                                          #
#   You should have received a copy of the GNU General Public License along#
#   with this program; if not, write to the Free Software Foundation, Inc.,#
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.            #
############################################################################

from soarapp import skip, parseFile, Container, AppManager
from DrawingWindow import killDrawingWindows
from Tkinter import *
from os import popen
from thread import start_new_thread

EDITOR_COMMAND = lambda fname: popen("emacs " + fname)
#EDITOR_COMMAND = lambda fname: popen("gnome-terminal -e \"vi " + fname + "\"")

class Brain:
	def __init__(self, parent, file):
		self.file = file
		envin = {}
		envin["robot"] = Container()
		envin["motorOutput"] = lambda v, w: parent.command("motorOutput", v, w)
		envin["pose"] = lambda: parent.command("pose")
		envin["sonarDistances"] = lambda: parent.command("sonarDistances")
		envin["sonarInfo"] = parent.command("sonarInfo")
		envin["readLog"] = lambda f: parent.command("readLog", f)
		envin["writeLog"] = lambda f: parent.command("writeLog", f)
		envin["discrete"] = lambda steptime: parent.command("discrete", steptime)
		envin["stalled"] = lambda: parent.command("stalled")
		envin["cheat"] = lambda: parent.command("cheat")
		envout = ["setup", "step"]
		returned = parseFile(file, envin, envout)
		self.setup = returned["setup"]
		self.step = returned["step"]
		self.win = AppManager.getWindow("Brain")
		Button(self.win, text = "Edit Brain", command = lambda: start_new_thread(EDITOR_COMMAND, (self.file,))).pack()
		Button(self.win, text = "Reload Brain", command = lambda: parent.brainButton(True)).pack()
		AppManager.exitcallbacks.append(self.kill)

	def destroy(self):
		AppManager.exitcallbacks.remove(self.kill)
		self.kill()

	def kill(self):
		killDrawingWindows()
		AppManager.killWindow("Brain")
