############################################################################
# interpreter.py - Michael Haimes                                          #
############################################################################
#    MIT SoaR 1.0 - A Python abstraction layer for MobileRobots            #
#    Pioneer3DX robots, and a simulator to simulate their operation        #
#    in a python environment (for testing)                                 #
#                                                                          #
#    Copyright (C) 2006 Massachusetts Institute of Technology              #
#      Written by Michael Haimes <mhaimes@mit.edu>                         #
#        with code from Ross Glashan <rng@mit.edu>                         #
#                                                                          #
#   This program is free software; you can redistribute it and/or modify   #
#   it under the terms of the GNU General Public License as published by   #
#   the Free Software Foundation; either version 2 of the License, or      #
#   (at your option) any later version.                                    #
#                                                                          #
#   This program is distributed in the hope that it will be useful,        #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
#   GNU General Public License for more details.                           #
#                                                                          #
#   You should have received a copy of the GNU General Public License along#
#   with this program; if not, write to the Free Software Foundation, Inc.,#
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.            #
############################################################################

from Tkinter import *
from string import split
from traceback import format_exception
import sys
from soarmath import clip
from soarapp import Container, skip, AppManager

PROMPT = ">>> "

class Interpreter:
    def __init__(self, parent):
        self.win = AppManager.getWindow("Interpreter")
        frame = InterpreterFrame(self.win, parent)
        frame.pack()
	self.kill = lambda: AppManager.killWindow("Interpreter")
	AppManager.exitcallbacks.append(self.kill)
        self.step = self.setup = skip

    def destroy(self):
	AppManager.exitcallbacks.remove(self.kill)
	self.kill()

class InterpreterFrame(Frame):

    def __init__(self, tkparent, parent):
        #borderwidth=2, relief=RIDGE
        Frame.__init__(self, tkparent)
        self.environment = {}
        self.cmdhistory = []
        self.curcmd = 0
        self.multilinecmd = []
        self.inmultilinecmd = False
        self.environment["robot"] = Container()
        self.environment["motorOutput"] = lambda v, w: parent.command("motorOutput", v, w)
        self.environment["pose"] = lambda: parent.command("pose")
        self.environment["sonarDistances"] = lambda: parent.command("sonarDistances")
        self.environment["sonarInfo"] = parent.command("sonarInfo")
        self.environment["readLog"] = lambda f: parent.command("readLog", f)
        self.environment["writeLog"] = lambda f: parent.command("writeLog", f)
        self.environment["discrete"] = lambda steptime: parent.command("discrete", steptime)
        self.environment["stalled"] = lambda: parent.command("stalled")
        self.environment["cheat"] = lambda: parent.command("cheat")
        #background = "black", foreground = "white"
        self.box = Text(self, height = 25, width = 80, wrap = CHAR)
        self.box.pack()
        #insertbackground = "white", background = "black", foreground = "white"
        self.entrybox = Entry(self, width = 80)
        self.entrybox.bind("<Return>", lambda e: self.entry())
        self.entrybox.bind("<Up>", lambda e: self.cycle(-1))
        self.entrybox.bind("<Down>", lambda e: self.cycle(1))
        self.entrybox.pack(side = BOTTOM)
        self.prompt()

#    def updateWorkingFile(self, expr):
#        self.workingfile.append(expr)
#        file = open("temp", "w")
#        for line in self.workingfile:
#            file.write(line + "\n")
#        file.close()

    def cycle(self, direction):
        self.curcmd = clip(self.curcmd+direction, -len(self.cmdhistory), -1)
        self.prompt(self.cmdhistory[self.curcmd])

    def prompt(self, init = ""):
        self.entrybox.delete(0, END)
#        if self.inmultilinecmd:
#            self.entrybox.insert(0, "... " + init)
#        else:
        self.entrybox.insert(0, PROMPT + init)
        self.entrybox.focus_force()

    def entry(self):
        expr = self.entrybox.get()
        for x in PROMPT.strip():
            expr = expr.strip(x)
        expr = expr.strip()
        self.cmdhistory.append(expr)
#        if len(expr) > 0 and expr[-1] == ":": self.inmultilinecmd = True
#        if self.inmultilinecmd:
#            self.box.insert(INSERT, "... " + expr + "\n")
#            if len(expr) == 0:
#                newexpr = ""
#                for line in self.multilinecmd:
#                    newexpr+=line
#                self.do(newexpr)
#                self.inmultilinecmd = False
#            else:
#                self.multilinecmd.append(expr + "\n")
#        else:
        self.box.insert(INSERT, PROMPT + expr + "\n")
        self.do(expr)
        self.box.see(INSERT)
        self.prompt()

    def do(self, expr):
        try:
            val = eval(expr, self.environment)
            self.box.insert(INSERT, `val` + "\n")
        except:
            try:
                exec expr in self.environment
            except:
		str = ""
		for line in format_exception(sys.exc_type, sys.exc_value, sys.exc_traceback)[3:]:
			str+=line
                self.box.insert(INSERT, str)
