############################################################################
# joystick.py - Michael Haimes                                             #
############################################################################
#    MIT SoaR 1.0 - A Python abstraction layer for MobileRobots            #
#    Pioneer3DX robots, and a simulator to simulate their operation        #
#    in a python environment (for testing)                                 #
#                                                                          #
#    Copyright (C) 2006 Massachusetts Institute of Technology              #
#      Written by Michael Haimes <mhaimes@mit.edu>                         #
#        with code from Ross Glashan <rng@mit.edu>                         #
#                                                                          #
#   This program is free software; you can redistribute it and/or modify   #
#   it under the terms of the GNU General Public License as published by   #
#   the Free Software Foundation; either version 2 of the License, or      #
#   (at your option) any later version.                                    #
#                                                                          #
#   This program is distributed in the hope that it will be useful,        #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
#   GNU General Public License for more details.                           #
#                                                                          #
#   You should have received a copy of the GNU General Public License along#
#   with this program; if not, write to the Free Software Foundation, Inc.,#
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.            #
############################################################################

from Tkinter import *
from soarapp import skip, AppManager

dim = (10, 10, 210, 210)
center = ((dim[2] + dim[0])/2, (dim[3] + dim[1])/2)
radius = ((dim[2] - dim[0])/2)**2
inside = lambda x, y: (center[0]-x)**2+(center[1]-y)**2 < radius 
transrot = lambda x,y: ((center[1]-y)/float(center[1]-dim[1]), (center[0]-x)/float(center[0]-dim[0]))	

class Joystick:

	def __init__(self, parent):
      		self.v = 0
      		self.w = 0
		self.win = AppManager.getWindow("Joystick")
      		self.canvas = Canvas(self.win, width = 220, height = 220, bg = 'white')

      		self.canvas.bind("<ButtonRelease-1>", self.canvas_clicked_up)
      		self.canvas.bind("<Button-1>", self.respond)
      		self.canvas.bind("<B1-Motion>", self.respond)
      		self.canvas.pack(side=TOP)

      		self.circle = self.canvas.create_oval(dim, fill = 'white')
      		self.canvas.create_oval(105, 105, 115, 115, fill='black')
      		self.setup = skip
		self.kill = lambda: AppManager.killWindow("Joystick")
		AppManager.exitcallbacks.append(self.kill)
		self.step = lambda: parent.command("motorOutput", self.v, self.w)

	def destroy(self):
		AppManager.exitcallbacks.remove(self.kill)
		self.kill()

	def clear(self):
    		for item in self.canvas.find_withtag("line"):
        		self.canvas.delete(item)
  
   	def canvas_clicked_up(self, event):
    		self.v, self.w = 0,0
    		self.clear()

   	def respond(self, event):
     		self.clear()  
     		if inside(event.x, event.y):
             		self.v, self.w = transrot(event.x, event.y)
             		self.canvas.create_line(110, 110, 110, event.y, fill = "red", width = 2, arrow = LAST, tags = "line")
             		self.canvas.create_line(110, 110, event.x, 110, fill = "blue", width = 2, arrow = LAST, tags = "line")
