############################################################################
# network.py - Michael Haimes                                              #
############################################################################
#    MIT SoaR 1.0 - A Python abstraction layer for MobileRobots            #
#    Pioneer3DX robots, and a simulator to simulate their operation        #
#    in a python environment (for testing)                                 #
#                                                                          #
#    Copyright (C) 2006 Massachusetts Institute of Technology              #
#      Written by Michael Haimes <mhaimes@mit.edu>                         #
#        with code from Ross Glashan <rng@mit.edu>                         #
#                                                                          #
#   This program is free software; you can redistribute it and/or modify   #
#   it under the terms of the GNU General Public License as published by   #
#   the Free Software Foundation; either version 2 of the License, or      #
#   (at your option) any later version.                                    #
#                                                                          #
#   This program is distributed in the hope that it will be useful,        #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
#   GNU General Public License for more details.                           #
#                                                                          #
#   You should have received a copy of the GNU General Public License along#
#   with this program; if not, write to the Free Software Foundation, Inc.,#
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.            #
############################################################################

from socket import *
from soarapp import alert, skip
from threading import Thread

ROBOT_PORT=50028

class NetRobotClient:

    def __init__(self, parent):
        alert("Warning: Net features are still highly experimental")
        self.parent = parent
        self.win = Toplevel(self.parent.app)
        self.win.wm_title(PROGRAM_NAME + ": NetClient")
        self.win.wm_iconbitmap(WINDOW_ICON)
        self.win.protocol('WM_DELETE_WINDOW', self.parent.trytoquit)
        self.servaddress = Entry(self.win, width = 20)
        self.servaddress.pack(side = "left")
        Button(self.win, text = "Connect", command = self.connect).pack(side = "right")
        self.step = skip
        self.setup = skip

    def destroy(self):
        try:
            self.listener.stop()
        except:
            pass
        self.win.destroy()

    def connect(self):
        sockobj = socket(AF_INET, SOCK_STREAM)
        sockobj.connect((self.servaddress.get(), ROBOT_PORT))
        self.listener = self.Listener()
        self.listener.setparams(sockobj, self.parent)
        self.listener.start()
    
    class Listener(Thread):

        def setparams(self, sockobj, parent):
            self.sockobj = sockobj
            self.parent = parent

        def run(self):
            while True:
                inc = `self.sockobj.recv(256)`.split("*")
                for data in inc[1:-1]:    
                    if data[:3] == "MO:":
                        v, w = data[3:].split(',')
                        v = float(v.strip())
                        w = float(w.strip())
                        self.parent.command("motorOutput",v,w)
                    elif data[:3] == "!SD":
                        newdata = `self.parent.command("sonarDistances")`
                        self.sockobj.send(newdata) 
                    elif data[:3] == "!PS":
                        newdata = `self.parent.command("pose")`
                        self.sockobj.send(newdata)

class NetRobotServer:

    def __init__(self, parent):
        alert("Warning: Net features are still highly experimental")
        self.parent = parent
        self.win = Toplevel(self.parent.app)
        self.win.wm_title(PROGRAM_NAME + ": NetServer")
        self.win.wm_iconbitmap(WINDOW_ICON)
        self.win.protocol('WM_DELETE_WINDOW', self.parent.trytoquit)
        Button(self.win, text = "Listen", command = self.startNetworkServices).pack(side = "top")
        Label(self.win, text = "Client Address: ").pack(side = "left")
        self.addresslabel = StringVar()
        Label(self.win, textvariable = self.addresslabel).pack(side = "right")
        self.v, self.w = 0, 0

    def destroy(self):
        try:
            self.connection.close()
        except:
            pass
        self.win.destroy()

    def startNetworkServices(self):
        try:
            self.connection.close()
        except:
            pass
        self.sockobj = socket(AF_INET, SOCK_STREAM)
        self.sockobj.bind(('', ROBOT_PORT))
        self.sockobj.listen(1)
        print "listening on port:", ROBOT_PORT
        self.connection, self.address = self.sockobj.accept()
        self.addresslabel.set(self.address)
        
    def motorOutput(self, v, w):
        if not(self.v == v and self.w == w):
            self.v, self.w = v, w
            data = "*MO:" + `v` + "," + `w`
            self.connection.send(data)    
    
    def sonarDistances(self):
        self.connection.send("!SD")
        data = self.connection.recv(2048)
        print "\"" + data + "\""
        return map(float, data[1:-1].split(','))    

    def pose(self):
        self.connection.send("!PS")
        data = self.connection.recv(2048)
        print "\"" + data + "\""
        return map(float, data[1:-1].split(','))    
