############################################################################
# runner.py - Michael Haimes                                               #
############################################################################
#    MIT SoaR 1.0 - A Python abstraction layer for MobileRobots            #
#    Pioneer3DX robots, and a simulator to simulate their operation        #
#    in a python environment (for testing)                                 #
#                                                                          #
#    Copyright (C) 2006 Massachusetts Institute of Technology              #
#      Written by Michael Haimes <mhaimes@mit.edu>                         #
#        with code from Ross Glashan <rng@mit.edu>                         #
#                                                                          #
#   This program is free software; you can redistribute it and/or modify   #
#   it under the terms of the GNU General Public License as published by   #
#   the Free Software Foundation; either version 2 of the License, or      #
#   (at your option) any later version.                                    #
#                                                                          #
#   This program is distributed in the hope that it will be useful,        #
#   but WITHOUT ANY WARRANTY; without even the implied warranty of         #
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the          #
#   GNU General Public License for more details.                           #
#                                                                          #
#   You should have received a copy of the GNU General Public License along#
#   with this program; if not, write to the Free Software Foundation, Inc.,#
#   51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.            #
############################################################################

from simulator import SONAR_INFO, MIN_STEPSIZE

from soarapp import skip, AppManager

from Tkinter import *
from tkFileDialog import askopenfilename
from tkMessageBox import askokcancel
from time import time
from time import sleep
from threading import Thread
from thread import start_new_thread
import pickle
import sys
try:
	from manual import manual as openManual
except ImportError:
	openManual = skip
	alert("You should run setup.py in the main directory")

DT_PER_STEP=0.05 #approximate time in seconds it takes per step.. used in the "Step" button

class Runner:
	
	def __init__(self):
		AppManager.setRunner(self)
		self.win = AppManager.getWindow("Main", True, True)
		self.controlpacklist = []
		self.flowpacklist = []
		
		self.menubar = Menu(self.win)
		filemenu = Menu(self.menubar, tearoff = 0)
		self.menubar.add_cascade(label = "File", menu = filemenu)
		filemenu.add_command(label = "Exit", command = AppManager.trytoquit)
		helpmenu = Menu(self.menubar, tearoff = 0)
		self.menubar.add_cascade(label = "Help", menu = helpmenu)
		helpmenu.add_command(label = "Read Manual...", command = openManual)
		self.windowmenu = Menu(self.menubar, tearoff = 0)
		self.menubar.add_cascade(label = "Window", menu = self.windowmenu)
		self.win.config(menu = self.menubar)
		
		Label(self.win, text= "Output:").pack(side = "top")
		outputframe = Frame(self.win)
		Button(outputframe, text= "Pioneer", command = self.pioneerButton).pack(side="left")
		Button(outputframe, text = "Simulator", command = self.simulatorButton).pack(side = "left")
	    #Button(outputframe, text = "NetServer", command = self.netserverButton).pack(side = "left")
		outputframe.pack(side = "top")
		self.controlpacklist.append((Label(self.win, text = "Control:"), "top"))
		controlframe = Frame(self.win)
		Button(controlframe, text = "Brain", command = self.brainButton).pack(side="left")
		Button(controlframe, text = "Joystick", command = self.joystickButton).pack(side = "left")
		Button(controlframe, text = "Interpreter", command = self.interpreterButton).pack(side = "left")
		#Button(controlframe, text = "NetClient", command = self.netclientButton).pack(side = "left")
		self.controlpacklist.append((controlframe, "top"))
		self.flowpacklist.append((Label(self.win, text = "Flow:"), "top"))
		flowframe = Frame(self.win)
		Button(flowframe, text = "Start", command = self.startall).pack(side = "left")
		Button(flowframe, text = "Step", command = self.stepall).pack(side = "left")
		Button(flowframe, text = "Stop", command = self.stopall).pack(side = "left")
		self.flowpacklist.append((flowframe, "top"))
		self.running = StringVar()
		self.running.set("Stopped")
		self.flowpacklist.append((Label(self.win, textvariable = self.running), "top"))
		self.readfile = self.writefile = False
		self.controllist = []
		self.addcontrol(self.startstepper, self.onestepper, self.stopstepper)
		AppManager.exitcallbacks.append(self.kill)
		try:
			AppManager.tk.mainloop()
		except KeyboardInterrupt:
			print "Exit Code -1: Control-C killed"
			AppManager.exit(-1)

	def destroy(self):
		AppManager.exitcallbacks.remove(self.kill)
		self.kill()

	def kill(self):
		self.stopstepper()
		AppManager.killWindow("Main")
	
	def brainButton(self, samebrain = False):
		from brain import Brain
		if not samebrain:
			self.brainfile = askopenfilename(title = "Open a Brain File...", initialdir = "brains")
		if len(self.brainfile) > 0:
			self.useControl(Brain(self, self.brainfile))
		if samebrain:
			print "Brain reloaded"
	def joystickButton(self):
		from joystick import Joystick
		self.useControl(Joystick(self))
	def interpreterButton(self):
		from interpreter import Interpreter
		self.useControl(Interpreter(self))
	def netclientButton(self):
		from network import NetRobotClient
		self.useControl(NetRobotClient(self))

	def pioneerButton(self):
		from pioneer import Pioneer
		self.useOutput(Pioneer(self))
		self.usingsim = False
	def simulatorButton(self):
		from simulator import Simulator
		self.worldfile = askopenfilename(title = "Open a World File...", initialdir = "worlds")
		if len(self.worldfile) > 0:
			self.useOutput(Simulator(self, self.worldfile))
			self.usingsim = True
	def netserverButton(self):
		from network import NetRobotServer
		self.useOutput(NetRobotServer(self))
		self.usingsim = False
		
	def command(self, str, first=0, second=0):
		if str == "sonarDistances":
			return self.storedsonarDistances
		elif str == "pose":
			return self.storedpose
		elif str == "sonarInfo":
			return SONAR_INFO
		elif str == "readLog":
			if self.writefile:
				alert("Don't be silly, you're already writing a log!")
			else:
				self.readfile = first
		elif str == "writeLog":
			if self.readfile:
				alert("Don't be silly, you're already reading a log!")
			else:
				self.writefile = first
		elif str == "motorOutput":
			self.output.motorOutput(first, second)
			if self.control.discretemode:
				sleep(self.control.discretetime)
				self.output.motorOutput(0,0)
		elif str == "discrete":
			self.control.discretemode = True
			self.control.discretetime = first
		elif str == "stalled":
			return self.output.stalled
		elif str == "cheat":
			return self.output
	def useControl(self, ctrlobj):
		try:
			self.control.destroy()
		except AttributeError:
			pass
		try:	
			self.stopall()
		except AttributeError:
			pass
		self.readfile = self.writefile = False
		self.control = ctrlobj
		self.control.discretemode = self.control.discretetime = False
		self.control.setup()
		if self.readfile:
			self.readfile = open(self.readfile, 'rb')
		if self.writefile:
			self.writefile = open(self.writefile, 'wb')
		for item in self.flowpacklist:
			item[0].pack(side = item[1])
		self.flowpacklist = []

	def useOutput(self, outobj):
		try:
			self.output.destroy()
		except AttributeError:
			pass
		try:
			self.stopall()
		except AttributeError:
			pass
		self.output = outobj
		self.logstep()
		for item in self.controlpacklist:
			item[0].pack(side = item[1])
		self.controlpacklist = []
		
	def logstep(self):
		t = time()
		if self.readfile:
			try:
				self.storedsonarDistances = pickle.load(self.readfile)
				self.storedpose = pickle.load(self.readfile)
			except EOFError:
				alert("Log Finished")
				self.stopall()
		else:
			self.storedsonarDistances = self.output.sonarDistances()
			self.storedpose = self.output.pose()
		if self.writefile:
			pickle.dump(self.storedsonarDistances, self.writefile)
			pickle.dump(self.storedpose, self.writefile)
	def startall(self):
		if self.control.discretemode or not self.usingsim:
			for i in self.controllist:
				i[0]()
		else:
			self.startstepper()

	def stepall(self, dt = 0):
		t = 0
		if dt:
			stepsize = dt
		else:
			t = time()
			if self.control.discretetime:
				stepsize = self.control.discretetime
			else:
   				stepsize = DT_PER_STEP
		for i in self.controllist:
			try:
				i[1](stepsize)
			except TclError:
				pass
		if t:
			if time()-t < stepsize:
				sleep(stepsize-time()+t)
			self.command("motorOutput") #stop robot from moving
	def stopall(self):
		try:
			if self.control.discretemode or not self.usingsim:
				for i in self.controllist:
					i[2]()
			else:
				self.stopstepper()
		except AttributeError:
			pass
		self.command("motorOutput") #stop robot from moving

	def addcontrol(self, start, step, stop):
		self.controllist.append((start, step, stop))
	
	def startstepper(self):
		self.stopstepper()
		self.currentthread = self.Stepper()
		self.logstep()
		self.currentthread.setparams(self.onestepper, self.stepall, self.control.discretemode, self.usingsim)
		self.currentthread.start()
		self.running.set("Running...")
	def stopstepper(self):
		try:
			self.currentthread.stop()
		except AttributeError:
			pass
		self.running.set("Stopped")

	def onestepper(self, dummyparameter):
		self.logstep()
		try:
			self.control.step()
		except:
			error("Caught an unhandled exception in the step() method")
			self.stopall()

	class Stepper(Thread):
		def setparams(self, onestep, stepall, discrete, usingsim):
			self.onestep = onestep
			self.stepall = stepall
			self.discrete = discrete
			self.usingsim = usingsim
		
		def run(self):
			self.keepgoing = True
			t = time()
			if self.discrete or not self.usingsim:
				while self.keepgoing:
					self.onestep(0)
			else:
				while self.keepgoing:
					dt = time()-t;t=time()
					if dt < MIN_STEPSIZE:
						sleep(MIN_STEPSIZE-dt)
						dt = MIN_STEPSIZE
					self.stepall(dt)
		def stop(self):
			self.keepgoing = False
